
let alphabet = "0123456789".split("");
//var alphabet = "abcdefghijklmnopqrstuvwxyz".split("");
let len = alphabet.length;
let updateInterval = 5000;
let canRun = true;

let svg = d3.select("svg"),
    width = +svg.attr("width"),
    height = +svg.attr("height");


//create a placeholder SVG element and center in the middle of the screen (along the Y-axis). The position of all of our text elements will be placed relative to this placeholder "g".
let centerOfScreenElement = svg.append("g").attr("transform", "translate(32," + ((height / 2) - 50) + ")");


function update(data) {

  let t = d3.transition().duration(updateInterval/2); //set the transition time for moving letters

  let existingData = centerOfScreenElement.selectAll("text").data();
  console.log("\n\n*** new data join! *** \n\n");
  console.log("existing data: " + existingData); //print the previously bound data
  console.log("incoming data: " + data); //print the incoming data
  

  // JOIN new data with old elements.
  let text = centerOfScreenElement.selectAll("text")
    .data(data, d => d); //compares incoming "data" with previous "data"

 
  // EXIT old elements not present in new data.
  text.exit()
    .attr("class", 
        d => {
          console.log("" + d + " will be exited..."); 

          return "exit"
        })
    .transition(t)
    .attr("y", 250) // move off-screen, down from center of screen
    .style("fill-opacity", 0.0)
    .remove();

  
  // ENTER new elements present in new data.
  text.enter().append("text")
    .text(function(d) { return d; })
    .attr("class", 
        d => {
          console.log("" + d + " will be entered...");  
          return "enter";
        }) 
    .attr("y", -250) 
    .attr("x", function(d, i) { return i * 32; })
    .style("fill-opacity", 0.0)
    .transition(t)
    .attr("y", 0) //move down, from off-screen to center of screen
    .style("fill-opacity", 1);
  
 
  // UPDATE old elements present in new data.
  text.attr("class", 
      d => {
        console.log("" + d + " will be updated...");  
        return "update";
      })
      .transition(t)
      .attr("x", function(d, i) { return i * 32; });

}



// The initial display.
update(alphabet);

// Grab a random sample of letters from the alphabet, in alphabetical order.
d3.interval(function() {
  if (canRun == true) {
    update(
      d3.shuffle(alphabet)
      .slice(0, Math.floor(Math.random() * len ))
      .sort()
      );
  }
},updateInterval);


// Make it so we can pause the animation
d3.select("svg")
.on("click", function() {
  canRun = !canRun;
  console.log("canRun = " + canRun);
});
